from flask import render_template
from flask import (
    render_template,
    request,
    redirect,
    url_for,
    flash,
    redirect,
    session,
)
from flask_login import (
    login_user,
    login_required,
    logout_user,
)
from forms import (
    RegistrationForm,
    LoginForm,
    ResetPasswordForm,
    UpdatePasswordForm,
    OptionsForm,
    UpdateUserForm,
    ReverifyEmailForm,
)
from utils import write_users_to_csv, create_initial_users, get_call_count, get_top_calls, get_unique_symbols, recreate_user_table
from mail_utils import  send_email, send_verification_email, resend_verification_email
from models import db, User, Call
from itsdangerous import SignatureExpired
from werkzeug.security import generate_password_hash, check_password_hash
from forms import RegistrationForm, LoginForm
from datetime import datetime, timezone


def index():
    #recreate_user_table(db)
    #create_initial_users(db)
    num_calls = get_call_count(db)
    num_symbols = get_unique_symbols(db)
    top_calls = get_top_calls(db, 5)
    return render_template("index.html", calls=top_calls, num_calls=num_calls, num_symbols=num_symbols)


def login():
    form = LoginForm()
    if form.validate_on_submit():
        user = User.query.filter_by(username=form.username.data).first()
        if user and check_password_hash(user.password, form.password.data):
            if user.confirmed:
                # Update last_login before logging in
                user.last_ip = request.remote_addr
                user.last_login = datetime.now(timezone.utc)
                user.login_count += 1
                db.session.commit()  # Commit the change
                print("IP=", user.last_ip)
                login_user(user, remember=form.remember)
                return redirect(url_for("dashboard"))
            else:
                flash("Please confirm your email address before logging in.", "warning")
                return redirect(url_for("resend_verification"))
        else:
            flash("Invalid username or password.", "danger")
    return render_template("login.html", title="Login", form=form)


@login_required
def logout():
    logout_user()
    return redirect(url_for("index"))


def register():
    form = RegistrationForm()
    if form.validate_on_submit():
        username = form.username.data
        email = form.email.data
        password = form.password.data
        hashed_password = generate_password_hash(password, method="sha256")
        existing_user = User.query.filter(
            (User.username == username) | (User.email == email)
        ).first()
        if existing_user:
            flash(
                "A user with that username or email already exists. Please choose another.",
                "warning",
            )
            return redirect(url_for("register"))

        new_user = User(username=username, email=email, password=hashed_password)
        db.session.add(new_user)
        db.session.commit()
        write_users_to_csv(db)
        send_verification_email(username,email)
        flash(
            "A confirmation email has been sent to your email address. Please check your inbox.",
            "info",
        )
        return redirect(url_for("login"))
    return render_template("register.html", title="Register", form=form)


@login_required
def edit_user(user_id):
    user = User.query.get(user_id)
    if not user:
        flash(f"User with ID {user_id} not found.", "danger")
        return redirect(url_for("dashboard"))
    form = UpdateUserForm()
    if request.method == "GET":
        form.username.data = user.username
        form.email.data = user.email
        form.email_confirmed.data = '1' if user.confirmed else '2' 
        # Populate the new fields
        form.login_count.data = user.login_count
        form.last_update.data = user.last_update
        form.last_login.data = user.last_login
        form.favorites.data = user.favorites

    if form.validate_on_submit():
        user.username = form.username.data
        user.email = form.email.data
        user.confirmed = (form.email_confirmed.data == '1')
        # Handle new fields
        user.last_update = form.last_update.data
        user.last_login = form.last_login.data
        user.favorites = form.favorites.data
        user.login_count = form.login_count.data

        db.session.commit()
        flash(f"User with ID {user_id} has been updated.", "success")
        return redirect(url_for("show_users"))
    return render_template("edit_user.html", form=form)


@login_required
def delete_user(user_id):
    user = User.query.get(user_id)
    if user:
        db.session.delete(user)
        db.session.commit()
        flash(f"User {user_id} has been deleted.", "success")
    else:
        flash(f"User {user_id} not found.", "danger")
    return redirect(url_for("show_users"))



from flask import request, redirect, url_for

def delete_selected_users():
    user_ids = request.form.getlist('user_ids') # Gets list of selected user IDs.
    if user_ids:
        for user_id in user_ids:
            try:
                user = User.query.get(int(user_id))  # Assuming User model exists
                if user:
                    db.session.delete(user)
            except ValueError: # Handle cases where user_id is not an integer
                pass # Or flash an error message
        db.session.commit()
    return redirect(url_for('show_users'))




def reset_password():
    form = ResetPasswordForm()
    if form.validate_on_submit():
        email = form.email.data
        user = User.query.filter_by(email=email).first()
        if user:
            subject = "Password Reset Email"
            token = s.dumps(email, salt="password-reset")
            body = f'Please click the link to reset your password: {url_for("update_password", token=token, _external=True)}'
            send_email(email, subject, body)
            flash(
                "A password reset email has been sent to your email address. Please check your inbox.",
                "info",
            )
            return redirect(url_for("login"))
        else:
            flash("No user found with that email address.", "danger")
    return render_template("reset_password.html", title="Login", form=form)


def update_password(token):
    from app import s
    try:
        email = s.loads(token, salt="password-reset", max_age=86400)
    except SignatureExpired:
        flash(
            "The password reset link has expired. Please request a new one.", "danger"
        )
        return redirect(url_for("reset_password.html"))
    form = UpdatePasswordForm()
    print("Email =", email)
    if form.validate_on_submit():
        user = User.query.filter_by(email=email).first()
        password = form.password.data
        hashed_password = generate_password_hash(password, method="sha256")
        user.password = hashed_password
        db.session.commit()
        flash("Your password has been updated. You can now log in.", "success")
        return redirect(url_for("login"))
    return render_template("update_password.html", form=form)


def resend_verification():
    form = ReverifyEmailForm()
    if form.validate_on_submit():
        resend_verification_email(form.email.data)
    else:
        print(form.errors)
    return render_template("reverify.html", title="Reverify Email", form=form)


def options():
    form = OptionsForm()
    if request.method == "GET":
        print("Model=", session["model"])
        form.model.data = session["model"]  # 1 = 'gpt-3.5-turbo', 2 = 'gpt-4'
        form.temperature.data = session["temperature"]
    if form.validate_on_submit():
        session["model"] = form.model.data
        session["temperature"] = float(form.temperature.data)
        flash("Options Saved!", "info")
        return redirect(url_for("dashboard"))
    return render_template("/GPT/options.html", form=form)
